import React from "react";
import PropTypes from "proptypes"
// 模块化 css, 后缀 .module.css
import SonComponentCss from "./SonComponent.module.css"
export default class SonComponent extends React.PureComponent {
    state = {
        msg: "SonComponent Msg",
        showChildrenBorder: true,
        childrenClass: []
    }

    // constructor(props) {
    //     super(props);
    //     console.log(props)
    // }

    componentDidMount() {
        this.changeChildrenClass()
        console.log('[挂载] SonComponent 组件后执行的操作');
    }

    componentDidUpdate(prevProps, prevState, snapshot) {
        // 当count更新时执行的操作
        console.log('SonComponent 组件后更新，执行的操作:', this.state);
    }


    componentWillUnmount() {
        console.log('[卸载] SonComponent 组件 前执行的操作');
    }

    switchChildrenBorder = () => {
        this.setState({
            ...this.state,
            showChildrenBorder: !this.state.showChildrenBorder
        }, ()=>{
            this.changeChildrenClass()
        })
    }

    changeChildrenClass = () => {
        let childrenClass = []
        if (this.state.showChildrenBorder) {
            childrenClass = [SonComponentCss.children]
        }

        this.setState({
            ...this.state,
            childrenClass
        })
    }

    render() {
        return <div style={{border: '1px solid', width: '80%', overflow: 'auto'}}>
            子组件

            <div>
                {this.state.msg}
            </div>

            <div>
                {this.props.msg}

                <div className={this.state.childrenClass}>
                    {this.props.children}
                </div>

                <div className={this.state.childrenClass}>
                    {this.props.slot2}
                </div>

                <div className={this.state.childrenClass}>
                    {this.props.scopeslot(<div>
                        <div>SonComponent 插槽 slot3 (作用域插槽)</div>
                        <div>
                            {this.state.msg}
                        </div>
                    </div>)}
                </div>

                <div>
                    <button onClick={this.switchChildrenBorder}>切换边框显示</button>
                </div>
            </div>
        </div>
    }
}

SonComponent.propTypes = {
    // 自定义类型校验
    // msg: (props)=>{
    //     if(typeof props.msg != "string"){
    //         throw new Error("msg 必须为 string 类型")
    //     }
    // }

    // 使用 proptypes 提供的类型校验
    msg: PropTypes.string
}

SonComponent.defaultProps = {
    msg: "defaultProps"
}
